// ForGet. A tool for managing development dependencies.
// Copyright (C) 2015  Daniel Hughes
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Xml.Linq;
using System.IO;
using System.Linq;

namespace ForGet
{
   public class Restorer
   {
      const string _localpath = "dependencies";
      const string _versionFile = "version.txt";
       
      public bool Restore(string configFile)
      {
         if(!File.Exists(configFile))
         {
            Console.Error.WriteLine("Error: Config file could not be found: {0}", configFile);
            return false;
         }
         if(!Directory.Exists(_localpath))
         {
            Directory.CreateDirectory(_localpath);  
         }
         XElement repo = XElement.Parse(File.ReadAllText(configFile));
         var repoPaths = ConfigHelpers.GetRepoPaths(repo).ToArray();
         if(!repoPaths.Select(path => path == null).Any())
         {
            return false;
         }

         foreach(var dependency in repo.Elements("Dependency"))
         {
            if(!RestoreDependency(dependency, new PackageRepository(repoPaths)))
            {
               return false;   
            }
         }
         return true;
      }
    
      bool RestoreDependency(XElement dependency, PackageRepository repo)
      {
         string id = dependency.Attribute("Id").Value;
         string version = dependency.Attribute("Version").Value;
       
         string toPath = Path.Combine(_localpath, id);
       
         if(Directory.Exists(toPath))
         {
            if(File.Exists(Path.Combine(toPath, _versionFile)) &&
                File.ReadAllText(Path.Combine(toPath, _versionFile)) == version)
            {
               Console.WriteLine("{0} {1} Already upto date.", id, version);
               return true;
            }
            Console.WriteLine("{0} Removing old version.", id, version);
            Directory.Delete(toPath, true);
         }
         Console.WriteLine("{0} {1} Restoring.", id, version);
         
         string pathFrom = repo.GetPackage(id, version);
         if (pathFrom == null)
         {
            Console.Error.WriteLine("Error: failed to find package {0} {1}", id, version);
            return false;
         }
         new Archive ().Install(pathFrom, toPath);
         File.WriteAllText(Path.Combine(toPath, _versionFile), version);
         
         foreach(var copy in dependency.Elements("Copy"))
         {
            DoCopy(copy, toPath);
         }
         return true;
      }
      
      string MakePathNative(string path)
      {
         return path
            .Replace("\\", Path.DirectorySeparatorChar.ToString())
            .Replace("/", Path.DirectorySeparatorChar.ToString());
      }
      
      void DoCopy(XElement copy, string packagePath)
      {
         string fromPath = Path.Combine(packagePath,MakePathNative(copy.Attribute("From").Value));
         string toPath = MakePathNative(copy.Attribute("To").Value);
         
         if(Directory.Exists(toPath))
         {
            Directory.Delete(toPath, true);  
         }
         
         //Now Create all of the directories
         Directory.CreateDirectory(toPath);
         foreach (string dirPath in Directory.GetDirectories(fromPath, "*", SearchOption.AllDirectories))
         {
             Directory.CreateDirectory(dirPath.Replace(fromPath, toPath));
         }
         
         //Copy all the files & Replaces any files with the same name
         foreach (string newPath in Directory.GetFiles(fromPath, "*.*", SearchOption.AllDirectories))
         {
             File.Copy(newPath, newPath.Replace(fromPath, toPath), true);
         }
      }
   }
}

